use axum::{
    extract::{Request, State},
    http::{header::LOCATION, HeaderMap, HeaderValue, StatusCode},
    middleware::Next,
    response::{IntoResponse, Response},
};
use axum_extra::extract::PrivateCookieJar;

use crate::BlogState;

/// Middleware layer requiring user to log in to access route
pub(crate) async fn logged_in(
    jar: PrivateCookieJar,
    State(state): State<BlogState>,
    request: Request,
    next: Next,
) -> Response {
    let next_uri = request.uri();
    let redirect_to_login = || -> Response {
        let mut headers = HeaderMap::new();
        headers.append(
            LOCATION,
            HeaderValue::from_str(&["/login", &next_uri.to_string()].concat()).unwrap(),
        );
        (StatusCode::FOUND, headers).into_response()
    };
    let db = state.db();
    let session_id = match jar.get("id") {
        Some(id) => match id.value_trimmed().parse::<i64>() {
            Ok(id) => id,
            Err(_) => {
                return redirect_to_login();
            }
        },
        None => {
            return redirect_to_login();
        }
    };
    if let Err(_error) = db.check_session(session_id).await {
        return redirect_to_login();
    }
    next.run(request).await
}
