use axum::{
    extract::{Path, State},
    http::{header::LOCATION, HeaderMap, HeaderValue, StatusCode},
    response::{Html, IntoResponse, Response},
    routing::get,
    Router,
};
use axum_extra::extract::PrivateCookieJar;

use crate::{html, BlogState};

pub(super) fn login(State(state): State<BlogState>) -> Router {
    Router::new()
        .route("/", get(login_page))
        .route("/*next", get(login_next))
        .with_state(state)
}

async fn login_next(
    jar: PrivateCookieJar,
    Path(path): Path<String>,
    State(state): State<BlogState>,
) -> Response {
    let db = state.db();
    match jar.get("id") {
        Some(id) => match id.value_trimmed().parse::<i64>() {
            Ok(id) => {
                if db.check_session(id).await.is_ok() {
                    return go_next(path).await;
                }
            }
            Err(_) => {
                {};
            }
        },
        None => {
            {};
        }
    }
    Html(html::login_status(&path, true)).into_response()
}

async fn login_page(jar: PrivateCookieJar, State(state): State<BlogState>) -> Response {
    let db = state.db();
    match jar.get("id") {
        Some(id) => match id.value_trimmed().parse::<i64>() {
            Ok(id) => {
                if db.check_session(id).await.is_ok() {
                    return go_next("/".to_string()).await;
                }
            }
            Err(_) => {
                {};
            }
        },
        None => {
            {};
        }
    }
    Html(html::login_status("/", true)).into_response()
}

async fn go_next(path: String) -> Response {
    let next = path.as_str();
    let mut headers = HeaderMap::new();
    let value = match HeaderValue::from_str(next) {
        Ok(value) => value,
        Err(_error) => {
            let mut headers = HeaderMap::new();
            headers.append(LOCATION, "/".parse().unwrap());
            return (StatusCode::FOUND, headers).into_response();
        }
    };
    headers.append(LOCATION, value);
    (StatusCode::FOUND, headers).into_response()
}
